/**
 * @file
 * VuoList_VuoHorizontalSide C type definition.
 *
 * @copyright Copyright © 2012–2017 Kosada Incorporated.
 * This code may be modified and distributed under the terms of the MIT License.
 * For more information, see http://vuo.org/license.
 */

#pragma once

#ifdef __cplusplus
extern "C"
{
#endif

/**
 * @ingroup VuoTypes
 * @defgroup VuoList_VuoHorizontalSide VuoList_VuoHorizontalSide
 * A list of @ref VuoHorizontalSide elements.
 *
 * @{
 */

// This header is generated by vuo/type/list/generateVariants.sh.

#if defined(DOXYGEN) || !defined(VuoList_VuoHorizontalSide_TYPE_DEFINED)
/// @{
/**
 * A list of @ref VuoHorizontalSide elements.
 */
typedef const struct VuoList_VuoHorizontalSide_struct { void *l; } * VuoList_VuoHorizontalSide;
/// @}
#endif

/**
 * Creates a new list of @ref VuoHorizontalSide elements.
 */
VuoList_VuoHorizontalSide VuoListCreate_VuoHorizontalSide(void);

/**
 * Creates a new list of `count` instances of `value`.
 *
 * Use this in conjunction with @ref VuoListGetData_VuoHorizontalSide to quickly initialize a large list.
 */
VuoList_VuoHorizontalSide VuoListCreateWithCount_VuoHorizontalSide(const unsigned long count, const VuoHorizontalSide value);

/**
 * Makes a shallow copy of `list` — its items are retained (not copied) by the new list.
 */
VuoList_VuoHorizontalSide VuoListCopy_VuoHorizontalSide(const VuoList_VuoHorizontalSide list);

/**
 * Returns the @ref VuoHorizontalSide at @c index.
 * Index values start at 1.
 * If the list has no items, returns a default value.
 * Attempting to access an out-of-bounds index returns the first item in the list (if the index is 0), or last item in the list (if the index is greater than the list size).
 */
VuoHorizontalSide VuoListGetValue_VuoHorizontalSide(const VuoList_VuoHorizontalSide list, const unsigned long index);

/**
 * Returns a pointer to a C array containing the list items.
 *
 * Use this if you need a fast way to get or change multiple list items.
 *
 * You can modify values in the list by changing them in this array.
 * Just don't attempt to access beyond the list size.
 *
 * The pointer becomes invalid if you modify the list size using other functions
 * (e.g., Insert, Prepend, Append, Cut, Remove);
 * if you use those functions, just get a new pointer by calling this function again.
 *
 * If the list has no items, returns NULL.
 *
 * The pointer remains owned by the list; don't free it.
 */
VuoHorizontalSide *VuoListGetData_VuoHorizontalSide(const VuoList_VuoHorizontalSide list);

/**
 * Changes the @ref VuoHorizontalSide at @c index.
 * Index values start at 1.
 *
 * If `expandListIfNeeded` is false: If the list has no items, nothing is changed.
 * Attempting to change an out-of-bounds index changes the first item in the list (if the index is 0), or last item in the list (if the index is greater than the list size).
 *
 * If `expandListIfNeeded` is true: If the list has less than `index` elements, it is expanded to accommodate the specified `index`.
 * If `index` is 0, the list is resized to contain 1 item set to `value`.
 */
void VuoListSetValue_VuoHorizontalSide(const VuoList_VuoHorizontalSide list, const VuoHorizontalSide value, const unsigned long index, bool expandListIfNeeded);

/**
 * Inserts the `VuoHorizontalSide` immediately before `index`.
 * Index values start at 1.
 * Inserting at index 0 prepends the value to the list.
 * Inserting at an index beyond the last value in the list appends the value to the list.
 */
void VuoListInsertValue_VuoHorizontalSide(const VuoList_VuoHorizontalSide list, const VuoHorizontalSide value, const unsigned long index);

/**
 * Prepends @c value to @c list.
 */
void VuoListPrependValue_VuoHorizontalSide(VuoList_VuoHorizontalSide list, const VuoHorizontalSide value);

/**
 * Appends @c value to @c list.
 */
void VuoListAppendValue_VuoHorizontalSide(VuoList_VuoHorizontalSide list, const VuoHorizontalSide value);

/**
 * Swaps the value at `indexA` with the value at `indexB`.
 */
void VuoListExchangeValues_VuoHorizontalSide(VuoList_VuoHorizontalSide list, const unsigned long indexA, const unsigned long indexB);

#ifdef VuoHorizontalSide_SUPPORTS_COMPARISON
/**
 * Sorts `list`.
 */
void VuoListSort_VuoHorizontalSide(VuoList_VuoHorizontalSide list);

/**
 * Returns true if the two lists are equivalent.
 *
 * NULL lists are never equal to non-NULL lists (even empty lists).
 */
bool VuoList_VuoHorizontalSide_areEqual(const VuoList_VuoHorizontalSide a, const VuoList_VuoHorizontalSide b);

/**
 * Returns true if `a` < `b`.
 */
bool VuoList_VuoHorizontalSide_isLessThan(const VuoList_VuoHorizontalSide a, const VuoList_VuoHorizontalSide b);
#endif

/**
 * Generates a random permutation of `list`.
 *
 * `chaos` ranges from 0 to 1.  When `chaos` is 1, a full Fisher–Yates shuffle is performed.  When less than 1, fewer iterations are performed.
 */
void VuoListShuffle_VuoHorizontalSide(VuoList_VuoHorizontalSide list, const double chaos);

/**
 * Reverses the order of the items in `list`.
 */
void VuoListReverse_VuoHorizontalSide(VuoList_VuoHorizontalSide list);

/**
 * Returns a new list containing items from the original list in the range specified by `startIndex` and `itemCount`.
 *
 * Items in the new list are retained (not copied) from the original list.
 */
VuoList_VuoHorizontalSide VuoListSubset_VuoHorizontalSide(VuoList_VuoHorizontalSide list, const signed long startIndex, const unsigned long itemCount);

/**
 * Removes the first value from @c list.
 */
void VuoListRemoveFirstValue_VuoHorizontalSide(VuoList_VuoHorizontalSide list);

/**
 * Removes the last value from @c list.
 */
void VuoListRemoveLastValue_VuoHorizontalSide(VuoList_VuoHorizontalSide list);

/**
 * Removes all values from @c list.
 */
void VuoListRemoveAll_VuoHorizontalSide(VuoList_VuoHorizontalSide list);

/**
 * Removes the `VuoHorizontalSide` at `index`.
 * Index values start at 1.
 * Attempting to remove index 0 or an index beyond the last value in the list has no effect.
 */
void VuoListRemoveValue_VuoHorizontalSide(VuoList_VuoHorizontalSide list, const unsigned long index);

/**
 * Returns the number of elements in @c list.
 */
unsigned long VuoListGetCount_VuoHorizontalSide(const VuoList_VuoHorizontalSide list);

/**
 * Decodes the JSON object @c js to create a new value.
 *
 * @eg{
 *	["uno", "dos", "tres", "catorce"]
 * }
 */
VuoList_VuoHorizontalSide VuoList_VuoHorizontalSide_makeFromJson(struct json_object * js);

/**
 * Encodes @c value as a JSON object.
 */
struct json_object * VuoList_VuoHorizontalSide_getJson(const VuoList_VuoHorizontalSide value);

/**
 * Produces a brief human-readable summary of @c value.
 */
char * VuoList_VuoHorizontalSide_getSummary(const VuoList_VuoHorizontalSide value);

/// @{
/**
 * Automatically generated function.
 */
VuoList_VuoHorizontalSide VuoList_VuoHorizontalSide_makeFromString(const char *str);
char * VuoList_VuoHorizontalSide_getString(const VuoList_VuoHorizontalSide value);
/// @}

/**
 * @}
 */

#ifdef __cplusplus
}
#endif
